<?php

namespace Drupal\eca_form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\eca\Event\BaseHookHandler;

/**
 * The handler for hook implementations within the eca_form.module file.
 */
class HookHandler extends BaseHookHandler {

  /**
   * Triggers the event to alter a form.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param string $form_id
   *   The form ID.
   */
  public function alter(array &$form, FormStateInterface $form_state, string $form_id): void {
    $this->triggerEvent->dispatchFromPlugin('form:form_build', $form, $form_state, $form_id);
    // Add the handler on class-level, to avoid expensive and possibly faulty
    // serialization of nested object references during form submissions.
    $form['#validate'][] = [static::class, 'validate'];
    $form['#submit'][] = [static::class, 'submit'];
    $this->addSubmitHandler($form);
  }

  /**
   * Add submit handler to nested elements if necessary.
   *
   * Walks through the element array recursively and adds the extra
   * submit-handler to all elements where necessary.
   *
   * @param array $elements
   *   A render array to walk through.
   */
  protected function addSubmitHandler(array &$elements): void {
    foreach (Element::children($elements) as $key) {
      if (is_array($elements[$key])) {
        if (isset($elements[$key]['#submit'])) {
          $elements[$key]['#submit'][] = [static::class, 'submit'];
        }
        $this->addSubmitHandler($elements[$key]);
      }
    }
  }

  /**
   * Triggers the event to validate a form.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public static function validate(array $form, FormStateInterface $form_state): void {
    \Drupal::service('eca_form.hook_handler')->triggerEvent
      ->dispatchFromPlugin('form:form_validate', $form, $form_state);
  }

  /**
   * Triggers the event to submit a form.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public static function submit(array $form, FormStateInterface $form_state): void {
    \Drupal::service('eca_form.hook_handler')->triggerEvent
      ->dispatchFromPlugin('form:form_submit', $form, $form_state);
  }

}
